@@ -1558,9 +1558,8 @@ class SamplerResult(object):
 
 class AbstractSampler(object):
 
-    def __init__(self, gp, sampler):
+    def __init__(self, gp):
         self.gp = gp
-        self.sampler = sampler
         self.pp = ParamaterPriors(gp.kern)
         self.ndim = len(gp.kern.optimizer_array)
 
@@ -1596,10 +1595,9 @@ class MCMCSampler(AbstractSampler):
 
         self.debug = debug
         self.n_walkers = n_walkers
-        self.ndim = len(gp.kern.optimizer_array)
 
-        sampler = emcee.EnsembleSampler(self.n_walkers, self.ndim, self.lnprob, moves=moves)
-        AbstractSampler.__init__(self, gp, sampler)
+        AbstractSampler.__init__(self, gp)
+        self.sampler = emcee.EnsembleSampler(self.n_walkers, self.ndim, self.lnprob, moves=moves)
 
     def lnprob(self, p):
         self.gp.kern.optimizer_array = p
@@ -1721,9 +1719,9 @@ class ParamaterPriors(object):
 class NestedSampler(AbstractSampler):
     
     def __init__(self, gp, nlive=500, bound='multi', sample='auto', **kargs_dynesty):
-        sampler = dynesty.NestedSampler(self.log_marginal_likelihood, self.prior_transform, self.ndim, nlive=nlive,
-                                        bound=bound, sample=sample, **kargs_dynesty)
-        AbstractSampler.__init__(self, gp, sampler)
+        AbstractSampler.__init__(self, gp)
+        self.sampler = dynesty.NestedSampler(self.log_marginal_likelihood, self.prior_transform, self.ndim, nlive=nlive,
+                                             bound=bound, sample=sample, **kargs_dynesty)
 
     def run(self, live_update=False, **kargs_dynesty):
         if live_update:
@@ -1785,17 +1783,11 @@ class NestedSampler(AbstractSampler):
 class UltraNestNestedSampler(AbstractSampler):
 
     def __init__(self, gp, nlive=500, bound='multi', **kargs_ultranest):
-        self.gp = gp
-        self.ndim = len(gp.kern.optimizer_array)
-        self.pp = ParamaterPriors(gp.kern)
-        self.n_p = 0
-        self.all_samples = []
-        self.all_lnlik = []
         self.nlive = nlive
         
-        sampler = ultranest.ReactiveNestedSampler(self.get_parameter_names(), self.log_marginal_likelihood,
-                                                  self.prior_transform, **kargs_ultranest)
-        AbstractSampler.__init__(self, gp, sampler)
+        AbstractSampler.__init__(self, gp)
+        self.sampler = ultranest.ReactiveNestedSampler(self.get_parameter_names(), self.log_marginal_likelihood,
+                                                       self.prior_transform, **kargs_ultranest)
 
     def run(self, live_update=False, **kargs_ultranest):
         viz_callback = 'auto'
