@@ -24,10 +24,15 @@ class Telescope(object):
     n_elements_per_stations = 1
     latitude = 0
     only_drift_mode = False
+    redundant_array = False
+    redundant_baselines = []
 
     def get_stat_pos_file(self):
         pass
 
+    def get_stat_pos(self):
+        return np.loadtxt(self.get_stat_pos_file())
+
     def get_sefd(self, freq):
         pass
 
@@ -39,8 +44,14 @@ class Telescope(object):
         lamb = const.c.value / freq
         return np.min([lamb ** 2 / 3., np.ones_like(lamb) * np.pi * distance_between_dipole ** 2 / 4.], axis=0)
 
+    def get_dish_aeff(self, freq, diameter, efficiency):
+        lamb = const.c.value / freq
+        return lamb ** 2 / (4 * np.pi) * efficiency * (np.pi * diameter / lamb) ** 2
+
     @staticmethod
     def from_name(name):
+        if name.startswith('hera'):
+            name.split('_')
         klasses = Telescope.__subclasses__()
         [klasses.extend(k.__subclasses__()) for k in klasses[:]]
 
@@ -146,7 +157,6 @@ class MWA1(Telescope):
     fov = 30
     du = 2
     pb_name = 'ant_4_1.05_gaussian'
-    n_elements_per_stations = 1
     latitude = -26.70122102627586
     only_drift_mode = False
 
@@ -162,6 +172,97 @@ class MWA1(Telescope):
         return 2 * const.k_B.value / a_eff * 1e26 * tsys
 
 
+class HERA(Telescope):
+    name = 'hera'
+    pb_name = 'ant_14_1.1_gaussian'
+    latitude = -30.72146
+    longitude = 21.42822
+    only_drift_mode = True
+    redundant_array = True
+    redundant_baselines = np.array([14.6, 25.28794179, 29.2, 38.62796914, 43.8, 
+                                    50.57588358, 52.64104862, 58.4, 63.63992458, 
+                                    66.90560515, 73., 75.86382537, 77.25593828, 
+                                    81.2893597, 87.6])
+    umin = 4
+    umax = 200
+    du = 1
+
+    def __init__(self, hex_num=11, split_core=True, sep=14.6):
+        self.hex_num = hex_num
+        self.split_core = split_core
+        self.sep = sep
+
+    def get_stat_pos(self):
+        positions = []
+        for row in range(self.hex_num - 1, -self.hex_num + self.split_core, -1):
+            # adding self.split_core deletes a row if it's true
+            for col in range(2 * self.hex_num - abs(row) - 1):
+                x_pos = self.sep * ((2 - (2 * self.hex_num - abs(row))) / 2 + col)
+                y_pos = row * self.sep * np.sqrt(3) / 2
+                positions.append([x_pos, y_pos, 0])
+                
+        # basis vectors (normalized to self.sep)
+        up_right = self.sep * np.asarray([0.5, np.sqrt(3) / 2, 0])
+        up_left = self.sep * np.asarray([-0.5, np.sqrt(3) / 2, 0])
+
+        # split the core if desired
+        if self.split_core:
+            new_pos = []
+            for pos in positions:
+                # find out which sector the antenna is in
+                theta = np.arctan2(pos[1], pos[0])
+                if pos[0] == 0 and pos[1] == 0:
+                    new_pos.append(pos)
+                elif -np.pi / 3 < theta < np.pi / 3:
+                    new_pos.append(np.asarray(pos) + (up_right + up_left) / 3)
+                elif np.pi / 3 <= theta < np.pi:
+                    new_pos.append(np.asarray(pos) + up_left - (up_right + up_left) / 3)
+                else:
+                    new_pos.append(pos)
+            # update the positions
+            positions = new_pos
+
+        return np.array(positions)
+
+    def get_sefd(self, freq, tsys_sky=60):
+        d = 14
+        eff = 0.78
+        trxc = 100
+
+        lamb = const.c.value / freq
+        a_eff = self.get_dish_aeff(freq, d, eff)
+        tsys = tsys_sky * lamb ** 2.55 + trxc
+        return 2 * const.k_B.value / a_eff * 1e26 * tsys
+
+
+class HERA56(HERA):
+    name = 'hera_56'
+
+    def __init__(self):
+        HERA.__init__(self, 5)
+
+
+class HERA120(HERA):
+    name = 'hera_120'
+
+    def __init__(self):
+        HERA.__init__(self, 7)
+
+
+class HERA208(HERA):
+    name = 'hera_208'
+
+    def __init__(self):
+        HERA.__init__(self, 9)
+
+
+class HERA320(HERA):
+    name = 'hera_320'
+
+    def __init__(self):
+        HERA.__init__(self, 11)
+
+
 class NenuFAR(Telescope):
 
     name = 'nenufar'
@@ -300,7 +401,7 @@ class TelescopeSimu(object):
 
         timev = np.arange(self.hal * 3600, self.har * 3600, self.timeres)
 
-        statpos = np.loadtxt(self.telescop.get_stat_pos_file())
+        statpos = self.telescop.get_stat_pos()
         nstat = statpos.shape[0]
 
         print('Simulating UV coverage ...')
@@ -328,12 +429,13 @@ class TelescopeSimu(object):
 
         for tt in timev:
             pr(i)
-            HA = (tt / 3600.) * (15. / 180) * np.pi - (6.8689389 / 180) * np.pi
-            dec = self.dec_deg * (np.pi / 180)
-            RM = np.matrix([[np.sin(HA), np.cos(HA), 0.0],
-                            [-np.sin(dec) * np.cos(HA), np.sin(dec) * np.sin(HA), np.cos(dec)],
-                            [np.cos(dec) * np.cos(HA), - np.cos(dec) * np.sin(HA), np.sin(dec)]])
-            statposuvw = np.dot(RM, statpos.T).T
+            ha_rad = (tt / 3600.) * (15. / 180) * np.pi
+            dec_rad = self.dec_deg * (np.pi / 180)
+            rot_mat = np.matrix([[np.sin(ha_rad), np.cos(ha_rad), 0.0],
+                            [-np.sin(dec_rad) * np.cos(ha_rad), np.sin(dec_rad) * np.sin(ha_rad), np.cos(dec_rad)],
+                            [np.cos(dec_rad) * np.cos(ha_rad), - np.cos(dec_rad) * np.sin(ha_rad), np.sin(dec_rad)]])
+
+            statposuvw = np.dot(rot_mat, statpos.T).T
             bu = m2a(statposuvw[b1, 0] - statposuvw[b2, 0])
             bv = m2a(statposuvw[b1, 1] - statposuvw[b2, 1])
             bw = m2a(statposuvw[b1, 2] - statposuvw[b2, 2])
@@ -354,20 +456,36 @@ class TelescopeSimu(object):
 
         return np.array(uu), np.array(vv), np.array(ww)
 
-    # def simu_ungridded_uv(self, uu_meter, vv_meter):
-    #     from ps_eor import psutil, datacube
-
-    #     meta = datacube.ImageMetaData.from_res(res, shape)
-    #     meta.wcs.wcs.cdelt[2] = psutil.robust_freq_width(self.freqs)
-    #     meta.set('PEINTTIM', self.timeres)
-    #     meta.set('PETOTTIM', (self.har - self.hal) * 3600)
-    #     w_cube = datacube.CartWeightCube(weights, g_uu, g_vv, self.freqs, meta)
-
-    #     return TelescopGriddedUV(w_cube, self)
-
-    def simu_gridded_uv(self, uu_meter, vv_meter, fov, oversampling_factor=4, min_weight=10):
+    def redundant_gridding(self, max_distance=1):
+        import sklearn.cluster
         from ps_eor import psutil, datacube
 
+        uu_meter, vv_meter, _ = self.simu_uv()
+
+        X = np.array([uu_meter, vv_meter]).T
+        c = sklearn.cluster.DBSCAN(eps=max_distance, min_samples=1)
+        c.fit(X)
+
+        c_id, idx, counts = np.unique(c.labels_, return_index=True, return_counts=True)
+
+        uu_meter_grid = uu_meter[idx]
+        vv_meter_grid = vv_meter[idx]
+
+        meta = datacube.ImageMetaData.from_res(0.01, (100, 100))
+        meta.wcs.wcs.cdelt[2] = psutil.robust_freq_width(self.freqs)
+        meta.set('PEINTTIM', self.timeres)
+        meta.set('PETOTTIM', (self.har - self.hal) * 3600)
+
+        w_cube = datacube.CartWeightsCubeMeter(np.repeat(counts[None, :], len(self.freqs), 0), 
+                                               uu_meter_grid, vv_meter_grid, self.freqs, meta)
+
+        return TelescopGridded(w_cube, self)
+
+    def image_gridding(self, fov, oversampling_factor=4, min_weight=10):
+        from ps_eor import psutil, datacube
+
+        uu_meter, vv_meter, _ = self.simu_uv()
+
         du = 1 / np.radians(fov)
         res = 1 / (oversampling_factor * self.umax)
         n_u = int(np.ceil(1 / (res * du)))
@@ -402,10 +520,10 @@ class TelescopeSimu(object):
         meta.set('PETOTTIM', (self.har - self.hal) * 3600)
         w_cube = datacube.CartWeightCube(weights, g_uu, g_vv, self.freqs, meta)
 
-        return TelescopGriddedUV(w_cube, self)
+        return TelescopGridded(w_cube, self)
 
 
-class TelescopGriddedUV(object):
+class TelescopGridded(object):
 
     def __init__(self, weights, telescope_simu):
         from ps_eor import psutil
@@ -419,9 +537,6 @@ class TelescopGriddedUV(object):
         self.weights.meta.update(self.telescope_simu.to_dict())
         self.weights.save(filename)
 
-    def get_slice(self, freq_start, freq_end):
-        return TelescopGriddedUV(self.weights.get_slice(freq_start, freq_end), self.telescope_simu)
-
     @staticmethod
     def load(filename):
         from ps_eor import datacube
@@ -429,7 +544,41 @@ class TelescopGriddedUV(object):
         weights = datacube.CartWeightCube.load(filename)
         telescope_simu = TelescopeSimu.from_dict(weights.meta.kargs, weights.freqs)
 
-        return TelescopGriddedUV(weights, telescope_simu)
+        if telescope_simu.telescop.redundant_array:
+            weights = datacube.CartWeightsCubeMeter(weights.data, weights.uu, weights.vv, weights.freqs,
+                                                    weights.meta, weights.uv_scale)
+
+        return TelescopGridded(weights, telescope_simu)
+
+    def get_tel_gridded_uv(self, freq_start=None, freq_end=None):
+        from ps_eor import datacube
+
+        if freq_start is None:
+            freq_start = self.weights.freqs[0]
+        if freq_end is None:
+            freq_end = self.weights.freqs[-1]
+
+        weights = self.weights.get_slice(freq_start, freq_end)
+
+        if isinstance(weights, datacube.CartWeightsCubeMeter):
+            m_freq = (freq_end + freq_start) / 2.
+            weights = weights.get_cube(m_freq)
+
+        return TelescopGriddedUV(weights, self.telescope_simu)
+
+
+class TelescopGriddedUV(object):
+
+    def __init__(self, weights, telescope_simu):
+        from ps_eor import psutil
+
+        self.weights = weights
+        self.telescope_simu = telescope_simu
+        self.name = self.telescope_simu.telescop.name
+        self.z = psutil.freq_to_z(self.weights.freqs.mean())
+
+    def get_slice(self, freq_start, freq_end):
+        return TelescopGriddedUV(self.weights.get_slice(freq_start, freq_end), self.telescope_simu)
 
     def get_sefd(self):
         return np.atleast_1d(self.telescope_simu.telescop.get_sefd(self.weights.freqs))
@@ -439,7 +588,12 @@ class TelescopGriddedUV(object):
 
         du = 0.75 / self.weights.meta.theta_fov
 
-        el = 2 * np.pi * (np.arange(self.weights.ru.min(), self.weights.ru.max(), du))
+        if self.telescope_simu.telescop.redundant_array:
+            mfreq = self.weights.freqs.mean()
+            b = self.telescope_simu.telescop.redundant_baselines
+            el = 2 * np.pi * b / (const.c.value / mfreq)
+        else:
+            el = 2 * np.pi * (np.arange(self.weights.ru.min(), self.weights.ru.max(), du))
 
         ps_conf = pspec.PowerSpectraConfig(el, window_fct='boxcar')
         ps_conf.filter_kpar_min = filter_kpar_min
