@@ -281,8 +281,7 @@ class PowerSpectraConfig(psutil.SimpleConfig):
         return config
 
     def copy(self):
-        new = PowerSpectraConfig()
-        new.config = self.config.copy()
+        new = psutil.SimpleConfig.copy(self)
         if self._el is not None:
             new._el = self._el.copy()
 
@@ -835,6 +834,7 @@ class PowerSpectraCart(BasePowerSpectra):
     def get_ps3d(self, kbins, data_cube, with_cov_err=True, weighted='default'):
         ''' Return spherically averaged power spectra of data_cube for kbins'''
         data_cube = self.eor.get_slice(data_cube)
+        data_cube.filter_uvrange(self.config.umin, self.config.umax)
 
         ps2d_norm = self.get_ps2d_norm(data_cube)
         ps_err_norm = self.get_ps_err_norm(data_cube)
@@ -870,6 +870,8 @@ class PowerSpectraCart(BasePowerSpectra):
         ''' Return spherically averaged power spectra of data_cube for kbins'''
         data_cube1 = self.eor.get_slice(data_cube1)
         data_cube2 = self.eor.get_slice(data_cube2)
+        data_cube1.filter_uvrange(self.config.umin, self.config.umax)
+        data_cube2.filter_uvrange(self.config.umin, self.config.umax)
 
         ps2d_norm = self.get_ps2d_norm(data_cube1)
         _, dft_cube2 = self.delay_transform(data_cube1)
@@ -1680,6 +1682,48 @@ class FourPanelPsResults(object):
             self.fig.savefig(filename, bbox_extra_artists=(self.lgd,), bbox_inches='tight')
 
 
+class ThreePanelPsResults(object):
+
+    def __init__(self, ps_gen, kbins, figsize=(10, 5), norm_factor=1, n_cols=1, dpi=100):
+        self.ps_gen = ps_gen
+        self.kbins = kbins
+        self.norm_factor = norm_factor
+        self.fig, self.axs = plt.subplots(ncols=n_cols, nrows=3, figsize=figsize, squeeze=False, sharey='row', dpi=dpi)
+        self.lgd = None
+
+    def add_cube(self, cube, label, ps_gen=None, col=0, **kargs):
+        if ps_gen is None:
+            ps_gen = self.ps_gen
+        (self.norm_factor * ps_gen.get_ps3d(self.kbins, cube)).plot(ax=self.axs[0, col], label=label, **kargs)
+        (self.norm_factor * ps_gen.get_ps2d(cube)).plot_kpar(ax=self.axs[1, col], label=label, **kargs)
+        (self.norm_factor * ps_gen.get_ps2d(cube)).plot_kper(ax=self.axs[2, col], label=label, **kargs)
+
+    def add_cube_ps_diff(self, cube1, cube2, label, ps_gen=None, col=0, **kargs):
+        if ps_gen is None:
+            ps_gen = self.ps_gen
+        (self.norm_factor * (ps_gen.get_ps2d(cube1) - ps_gen.get_ps2d(cube2))).plot_kpar(ax=self.axs[0, col], label=label, **kargs)
+        (self.norm_factor * (ps_gen.get_ps2d(cube1) - ps_gen.get_ps2d(cube2))).plot_kper(ax=self.axs[1, col], label=label, **kargs)
+        (self.norm_factor * ps_gen.get_ps3d_with_noise(self.kbins, cube1, cube2)).plot(ax=self.axs[2, col], label=label, **kargs)
+
+    def add_ps_stacker(self, ps_stacker, label, col=0, **kargs):
+        (self.norm_factor * ps_stacker.get_ps3d()).plot(ax=self.axs[0, col], label=label, **kargs)
+        (self.norm_factor * ps_stacker.get_ps2d()).plot_kpar(ax=self.axs[1, col], label=label, **kargs)
+        (self.norm_factor * ps_stacker.get_ps2d()).plot_kper(ax=self.axs[2, col], label=label, **kargs)
+
+    def done(self, ncol_legend=3, col_legend=0, **kargs):
+        self.lgd = self.fig.legend(*self.axs[0, col_legend].get_legend_handles_labels(),
+                                   bbox_to_anchor=(0.5, 1.04), loc="upper center", ncol=ncol_legend, **kargs)
+        for ax in self.axs[:, 1:].flatten():
+            ax.set_ylabel('')
+        self.fig.tight_layout()
+
+    def savefig(self, filename, **kargs):
+        if self.lgd is None:
+            print('Warning: run done() before savefig()')
+        else:
+            self.fig.savefig(filename, bbox_extra_artists=(self.lgd,), bbox_inches='tight')
+
+
 class MultiNight2DPowerSpectra(PowerSpectraMath):
 
     def __init__(self, ps, err, x, xlabel, y, ylabel):
@@ -1815,7 +1859,7 @@ class SphericalPowerSpectraMC(PowerSpectraMC, SphericalPowerSpectra):
         ax.set_yscale('log', **nonpos_arg)
         ax.set_xscale('log')
 
-        ax.set_ylabel(r'$\Delta^2 (k)\,[\mathrm{%s}]$' % temp_unit)
+        ax.set_ylabel(r'$\Delta^2 (k)\,[\mathrm{%s}^2]$' % temp_unit)
         ax.set_xlabel(r'$k\,[\mathrm{h\,cMpc^{-1}}]$')
 
         ax.set_xlim(self.k_bins.min(), self.k_bins.max())
