@@ -7,6 +7,7 @@ import os
 import matplotlib
 matplotlib.use('Agg')
 
+from ps_eor import __version__
 from ps_eor import datacube, psutil
 
 import numpy as np
@@ -59,7 +60,7 @@ def make_rnd_uvfits(tmp_dir, nf, shape, res):
     return f1, fits, uu, vv, data
 
 
-def make_rnd_imgfits(tmp_dir, df, shape, res, psf=False, n_chann=1, freq_width=None, wscnormf=1):
+def make_rnd_imgfits(tmp_dir, df, shape, res, psf=False, freq_width=None, wscnormf=1):
     f1 = np.arange(100, 110, df) * 1e6
 
     if freq_width is None:
@@ -69,11 +70,10 @@ def make_rnd_imgfits(tmp_dir, df, shape, res, psf=False, n_chann=1, freq_width=N
     wcs.wcs.crpix = np.array([0, 0, 1, 1])
     wcs.wcs.crval = [0, 0, 1, 1]
     wcs.wcs.ctype = ['RA---SIN', 'DEC--SIN', 'FREQ', 'STOKES']
-    wcs.wcs.cdelt = [-np.degrees(res), np.degrees(res), freq_width, 1]
+    wcs.wcs.cdelt = [-np.degrees(res), np.degrees(res), freq_width * 1e6, 1]
     wcs.wcs.cunit = ['deg', 'deg', 'Hz', '']
 
     if psf:
-        print(randn(1))
         fits = [str(tmp_dir / ('f%.3f_psf.fits' % f)) for f in f1]
         data = psutil.vis_to_img(10 + abs(randn(len(f1), shape[0], shape[1])), axes=(1, 2)).real
     else:
@@ -82,7 +82,7 @@ def make_rnd_imgfits(tmp_dir, df, shape, res, psf=False, n_chann=1, freq_width=N
 
     for i in range(len(f1)):
         wcs.wcs.crval = [0, 0, f1[i], 1]
-        hdu = datacube.pf.PrimaryHDU(data[i][:, :, None, None])
+        hdu = datacube.pf.PrimaryHDU(data[i][None, None, :, :])
         hdu.header['PETOTTIM'] = 20
         hdu.header['PEINTTIM'] = 2
         hdu.header['DATE-OBS'] = '2010-08-10'
@@ -92,7 +92,6 @@ def make_rnd_imgfits(tmp_dir, df, shape, res, psf=False, n_chann=1, freq_width=N
         else:
             hdu.header['BMAJ'] = 3 * res
             hdu.header['BMIN'] = 3 * res
-        hdu.header['PENCHANN'] = n_chann
         hdu.header.update(wcs.to_header())
         hdu.writeto(fits[i], overwrite=True)
 
@@ -240,7 +239,7 @@ def test_image_meta_data():
     assert np.allclose(meta.wcs.wcs.crpix, (50, 50))
 
     assert meta.to_header()['PETOTTIM'] == 20
-    assert meta.to_header(add_origin=True)['ORIGIN'] == 'ps_eor v_%s' % datacube.__version__
+    assert meta.to_header(add_origin=True)['ORIGIN'] == 'ps_eor v_%s' % __version__
 
 
 def test_image_cube(tmp_path):
@@ -496,8 +495,8 @@ def test_load_from_fits_image_and_psf(tmp_path):
     assert np.allclose(cube.meta.res, 1e-3)
     assert cube.meta.int_time == 2
     assert cube.meta.total_time == 20
-    assert np.allclose(cube.meta.freq_width, 0.4)
-    assert np.allclose(cube.weights.meta.freq_width, 0.4)
+    assert np.allclose(cube.meta.freq_width, 0.4e6)
+    assert np.allclose(cube.weights.meta.freq_width, 0.4e6)
 
     cube_h = datacube.CartDataCube.load_from_fits_image_and_psf(fits, fits_psf, 100, 500, np.radians(4),
                                                                 window_function=datacube.WindowFunction(
@@ -507,14 +506,14 @@ def test_load_from_fits_image_and_psf(tmp_path):
     assert (cube_h.meta.win_fct.name, cube_h.meta.win_fct.circular) == ('hann', True)
 
     np.random.seed(0)
-    f1, fits_n3, data_n3 = make_rnd_imgfits(tmp_path, 0.5, (100, 100), 1e-3, n_chann=3, freq_width=0.4)
-    _, fits_psf_n3, data_psf_n3 = make_rnd_imgfits(tmp_path, 0.5, (100, 100), 1e-3, psf=True, n_chann=3, freq_width=0.4)
+    f1, fits_n3, data_n3 = make_rnd_imgfits(tmp_path, 0.5, (100, 100), 1e-3, freq_width=0.4 / 3.)
+    _, fits_psf_n3, _ = make_rnd_imgfits(tmp_path, 0.5, (100, 100), 1e-3, psf=True, freq_width=0.4 / 3.)
 
     cube_n3 = datacube.CartDataCube.load_from_fits_image_and_psf(fits_n3, fits_psf_n3, 100, 500, np.radians(4))
 
     assert np.allclose(data_n3[0, 0], data[0, 0])
     assert np.allclose(cube.data[0, 0], cube_n3.data[0, 0])
-    assert np.allclose(cube.weights.data[0, 0], 3 * cube_n3.weights.data[0, 0])
+    assert np.allclose(cube.weights.data[0, 0], cube_n3.weights.data[0, 0])
     assert np.allclose(cube.estimate_sefd(), np.sqrt(3) * cube_n3.estimate_sefd())
     assert np.allclose(cube.estimate_uv_sefd().data, np.sqrt(3) * cube_n3.estimate_uv_sefd().data)
     assert np.allclose(cube.estimate_freqs_sefd(), np.sqrt(3) * cube_n3.estimate_freqs_sefd())
