@@ -1061,7 +1061,9 @@ def validate_dec_deg(ctx, param, value):
 
 
 @main.command('simu_uv')
-@click.argument('instru', type=click.Choice(['ska_low', 'nenufar', 'nenufar_80', 'nenufar_52', 'lofar_hba', 'a12_hba', 'a12_lba', 'mwa1'], case_sensitive=False))
+@click.argument('instru', type=click.Choice(['ska_low', 'nenufar', 'nenufar_80', 'nenufar_52', 'lofar_hba', 'a12_hba',
+                                             'a12_lba', 'mwa1', 'hera', 'hera_56', 'hera_120', 'hera_208', 'hera_320'], 
+                                             case_sensitive=False))
 @click.argument('z', type=FloatRangeType())
 @click.option('--out_filename', '-o', help='Base filename of the output file', type=str, default='simu_uv',
               show_default=True)
@@ -1113,18 +1115,6 @@ def simu_uv(instru, z, out_filename, umin, umax, dec_deg, fov, bandwidth, df, to
     telescop_simu = obssimu.TelescopeSimu(telescop, freqs, dec_deg, - total_time / 2, total_time / 2, umin,
                                           umax, int_time, not allow_intra_baselines)
 
-    if fov is None:
-        fov = telescop.fov
-    elif fov == 'pb_fwhm':
-        pb = datacube.PrimaryBeam.from_name(telescop.pb_name)
-        fov = np.degrees(pb.get_fwhm(freqs.mean()))
-    else:
-        try:
-            fov = float(fov)
-        except Exception:
-            print('fov should be a float or "pb_fwhm')
-            return 1
-
     print(f'Computing gridded UV configuration for instrument {telescop.name}')
     print(f'Simulating {total_time} hour long observation with {int_time} s integration time')
     print(f'Pointing : {dec_deg:.1f} deg, HA between {- total_time / 2:.2f} hr and {total_time / 2:.2f} hr')
@@ -1132,19 +1122,33 @@ def simu_uv(instru, z, out_filename, umin, umax, dec_deg, fov, bandwidth, df, to
     print(f'{len(freqs)} frequency channels between {freqs.min() * 1e-6:.1f} and {freqs.max() * 1e-6:.1f} MHz')
     print(f'Frequency resolution: {df:.1f} kHz')
     print(f'Baseline range: {telescop_simu.umin} - {telescop_simu.umax} lambda (discarding intra-baselines: {telescop_simu.remove_intra_baselines})')
-    print(f'FoV: {fov:.1f} degrees')
 
-    uu_meter, vv_meter, _ = telescop_simu.simu_uv()
-
-    if res is not None:
-        osamp_factor = int(np.round((1 / telescop_simu.umax) / np.radians(6 / 60.)))
+    if telescop.redundant_array:
+        uv_simu = telescop_simu.redundant_gridding()
+        print(f'Redundant baselines: {uv_simu.weights.data.shape[1]}')
     else:
-        osamp_factor = 4
+        if fov is None:
+            fov = telescop.fov
+        elif fov == 'pb_fwhm':
+            pb = datacube.PrimaryBeam.from_name(telescop.pb_name)
+            fov = np.degrees(pb.get_fwhm(freqs.mean()))
+        else:
+            try:
+                fov = float(fov)
+            except Exception:
+                print('fov should be a float or "pb_fwhm')
+                return 1
 
-    uv_simu = telescop_simu.simu_gridded_uv(uu_meter, vv_meter, fov, osamp_factor, min_weight=0)
+        if res is not None:
+            osamp_factor = int(np.round((1 / telescop_simu.umax) / np.radians(6 / 60.)))
+        else:
+            osamp_factor = 4
 
-    print(f'Image size: {uv_simu.weights.meta.shape}')
-    print(f'Image resolution: {np.degrees(uv_simu.weights.meta.res) * 60:.2f} arcmin (oversampling: {osamp_factor})')
+        uv_simu = telescop_simu.image_gridding(fov, osamp_factor, min_weight=0)
+
+        print(f'FoV: {fov:.1f} degrees')
+        print(f'Image size: {uv_simu.weights.meta.shape}')
+        print(f'Image resolution: {np.degrees(uv_simu.weights.meta.res) * 60:.2f} arcmin (oversampling: {osamp_factor})')
 
     name = f'{out_filename}_{telescop.name}_z{z_str}_{total_time}h'
 
@@ -1162,7 +1166,7 @@ def simu_uv(instru, z, out_filename, umin, umax, dec_deg, fov, bandwidth, df, to
 
 @main.command('simu_noise_img')
 @click.argument('uv_simu_h5', type=t_file)
-@click.argument('total_time_hour', type=int)
+@click.argument('total_time_hour', type=float)
 @click.option('--out_filename', '-o', help='Base filename of the output file', type=str, default='simu_noise')
 @click.option('--min_weight', help='Minimum number of visibilities in a uv-cell', default=10, type=int, show_default=True)
 def simu_noise_img(uv_simu_h5, total_time_hour, out_filename, min_weight):
@@ -1170,8 +1174,13 @@ def simu_noise_img(uv_simu_h5, total_time_hour, out_filename, min_weight):
 
     from ps_eor import datacube, psutil, obssimu
 
-    uv_simu = obssimu.TelescopGriddedUV.load(uv_simu_h5)
-    freqs = uv_simu.weights.freqs
+    simu_gridded = obssimu.TelescopGridded.load(uv_simu_h5)
+    freqs = simu_gridded.weights.freqs
+    uv_simu = simu_gridded.get_tel_gridded_uv(freqs[0], freqs[-1])
+
+    if uv_simu.telescope_simu.telescop.redundant_array:
+        print('Image noise cube simulation not implemented for redundant arrays')
+        return
 
     print(uv_simu.weights.ru.min(), uv_simu.weights.ru.max(), np.diff(uv_simu.weights.uu)[0])
 
@@ -1206,8 +1215,10 @@ def simu_noise_ps(uv_simu_h5, total_time_hour, out_filename, kmin, kmax, nks,
 
     from ps_eor import datacube, psutil, obssimu
 
-    uv_simu = obssimu.TelescopGriddedUV.load(uv_simu_h5)
-    freqs = uv_simu.weights.freqs
+    simu_gridded = obssimu.TelescopGridded.load(uv_simu_h5)
+    freqs = simu_gridded.weights.freqs
+    uv_simu = simu_gridded.get_tel_gridded_uv(freqs[0], freqs[-1])
+
     ps_gen = uv_simu.get_ps_gen(filter_kpar_min, filter_wedge_theta)
     telescop = uv_simu.telescope_simu.telescop
     telescop_simu = uv_simu.telescope_simu
@@ -1269,17 +1280,18 @@ def simu_noise_ps(uv_simu_h5, total_time_hour, out_filename, kmin, kmax, nks,
 @click.option('--filter_wedge_theta', help='Filter mode below the wedge line', default=0, type=float)
 @click.option('--min_weight', help='Minimum number of visibilities in a uv-cell', default=10, type=int, show_default=True)
 @click.option('--smooth', help='Smooth the resulting sensitivity curve', is_flag=True)
+@click.option('--n_incoherent_avg', help='Number of incoherent averaging', default=1, type=int)
 # allow for multiple uv_simu_h5
 # add k_min
 def simu_noise_ps_zrange(uv_simu_h5, total_time_hour, k, out_filename, bandwidth, delta_z, dk_over_k,
-                         filter_kpar_min, filter_wedge_theta, min_weight, smooth):
+                         filter_kpar_min, filter_wedge_theta, min_weight, smooth, n_incoherent_avg):
     ''' Compute noise PS from a simulated UV coverage UV_SIMU_H5 for TOTAL_TIME_HOUR observation at k of K.'''
     from ps_eor import psutil, obssimu
     from scipy.signal import savgol_filter
 
-    uv_simu = obssimu.TelescopGriddedUV.load(uv_simu_h5)
-    z_max = np.round(psutil.freq_to_z(uv_simu.weights.freqs.min() + bandwidth / 2 * 1e6), 1)
-    z_min = np.round(psutil.freq_to_z(uv_simu.weights.freqs.max() - bandwidth / 2 * 1e6), 1)
+    simu_gridded = obssimu.TelescopGridded.load(uv_simu_h5)
+    z_max = np.round(psutil.freq_to_z(simu_gridded.weights.freqs.min() + bandwidth / 2 * 1e6), 1)
+    z_min = np.round(psutil.freq_to_z(simu_gridded.weights.freqs.max() - bandwidth / 2 * 1e6), 1)
     zs = np.arange(z_min, z_max + delta_z, delta_z)
 
     k_bin = [(1 - dk_over_k / 2) * k, (1 + dk_over_k / 2) * k]
@@ -1291,7 +1303,7 @@ def simu_noise_ps_zrange(uv_simu_h5, total_time_hour, k, out_filename, bandwidth
 
     for z in zs:
         mfreq = psutil.z_to_freq(z)
-        uv_simu_z = uv_simu.get_slice(mfreq - bandwidth / 2 * 1e6, mfreq + bandwidth / 2 * 1e6)
+        uv_simu_z = simu_gridded.get_tel_gridded_uv(mfreq - bandwidth / 2 * 1e6, mfreq + bandwidth / 2 * 1e6)
 
         freqs = uv_simu_z.weights.freqs
         ps_gen = uv_simu_z.get_ps_gen(filter_kpar_min, filter_wedge_theta)
@@ -1303,6 +1315,9 @@ def simu_noise_ps_zrange(uv_simu_h5, total_time_hour, k, out_filename, bandwidth
         noise_std = uv_simu_z.get_noise_std_cube(total_time_hour * 3600, min_weight=min_weight)
 
         ps3d = ps_gen.get_ps3d(k_bin, noise_std)
+        if n_incoherent_avg > 1:
+            ps3d.err = ps3d.err / np.sqrt(n_incoherent_avg)
+
         pks.append(ps3d.data[0] * 1e6)
         pks_err.append(ps3d.err[0] * 1e6)
 
@@ -1315,11 +1330,11 @@ def simu_noise_ps_zrange(uv_simu_h5, total_time_hour, k, out_filename, bandwidth
         pks_err = 10 ** savgol_filter(np.log10(pks_err), w_l, 1)
 
     array_data = np.array([zs, [k_bin[0]] * len(zs), [k_bin[1]] * len(zs), [k] * len(zs), pks, pks_err]).T
-    header = f'Spherically averaged power-spectra {uv_simu.name} {total_time_hour}h\n'
+    header = f'Spherically averaged power-spectra {simu_gridded.name} {total_time_hour}h\n'
     header += f'Parameters: BW={bandwidth:.1f} MHz, kpar_min={filter_kpar_min}, filter_wedge={filter_wedge_theta} deg, dk/k={dk_over_k}\n'
     header += (r'z, k_min (h cMpc^-1), k_max (h cMpc^-1), k_mean (h cMpc^-1), \Delta^2 (mK^2), \Delta_err^2 (mK^2)')
 
-    name = f'{out_filename}_ps_{uv_simu.name}_z{z_min:.1f}-{z_max:.1f}_{total_time_hour}h_k{k}'
+    name = f'{out_filename}_ps_{simu_gridded.name}_z{z_min:.1f}-{z_max:.1f}_{total_time_hour}h_k{k}'
     np.savetxt(f'{name}.txt', array_data, fmt='%14.4f', header=header, delimiter=' ')
 
     fig, ax = plt.subplots(figsize=(5, 4))
